# -*- coding: utf8 -*-

"""
Test-Runner für die Tests gegen standalone
"""

def runtests():
    """
    The test runner for setup.py test usage. This sets up
    a memory database with sqlite3 and runs the tests via
    the django test command.
    """
    from conf import settings
    settings = settings(
        DATABASE_ENGINE='sqlite3',
        DATABASE_NAME=':memory:',
    )

    from django.test.utils import get_runner
    test_runner = get_runner(settings)
    failures = test_runner([], verbosity=1, interactive=True)
    raise SystemExit(failures)




# -*- coding: utf8 -*-

"""
A very simple metaclass to define models in a standalone script
and have those models show up in the standalone.models package.
This allows writing standalone scripts and use the Django ORM
for Database access without jumping through the hoops of
multiple models.

This hack is needed because most django tools need some kind of
installed applications to work. standalone functions as a dummy
application that is patched into dummy dynamic settings at
runtime to make it work.
"""

from django.db.models import *
from django.db.models import base
from django.utils.importlib import import_module

class StandaloneModelBase(base.ModelBase):
    """
    The metaclass for standalone models. This metaclass pushes
    all derived models into the standalone apps models package
    to spoof the management commands and all other tools
    to think there are really models in this package.
    """
    def __new__(cls, name, bases, dct):
        def getMeta(k, default=None):
            try:
                return getattr(dct['Meta'], k)
            except KeyError:
                return default
            except AttributeError:
                return default

        is_script_model = dct.get('__module__', '__main__') == '__main__'
        if not getMeta('abstract'):
            dct['__module__'] = 'standalone.models'
        newClass = base.ModelBase.__new__(cls, name, bases, dct)
        if is_script_model or dct.get('force_install_standalone_models', False):
            mod = import_module('standalone.models')
            setattr(mod, name, newClass)
        return newClass

class StandaloneModel(Model):
    """
    This class is the base class your own models will be
    derived from. Use it just as you would use models.Model
    in a normal Django project.
    """
    __metaclass__ = StandaloneModelBase

    class Meta:
        abstract = True




# -*- coding: utf8 -*-

from django.test import TestCase

"""
Testcases for django-standalone. Run with the provided test.py script.
"""

class DjangoStandaloneTests(TestCase):

    def setUp(self):
        # build some test models we need for test cases
        from django.core.management import call_command
        from standalone import models
        
        class MyModel(models.StandaloneModel):

            force_install_standalone_models = True

            col1 = models.CharField(max_length=1000)
            col2 = models.IntegerField()
            col3 = models.BooleanField()

            def __unicode__(self):
                return self.col1

        class MyOtherModel(models.StandaloneModel):

            col1 = models.CharField(max_length=100)
            col2 = models.ForeignKey("standalone.MyModel")

        call_command('syncdb')

    def test_environment(self):
        """
        Just make sure that all is set up correctly.
        """
        self.assert_(True)

    def test_namespaces(self):
        """
        test that models are in the right namespaces
        (or are not in the right namespaces)
        """
        import standalone.models
        self.assert_(hasattr(standalone.models, 'MyModel'))
        self.assert_(not(hasattr(standalone.models, 'MyOtherModel')))

    def test_create_objects(self):
        """
        Create a few objects in the database and check
        that they are actually where they belong.
        """
        from models import MyModel
        o1 = MyModel(col1='foo', col2=1, col3=True)
        o1.save()
        o2 = MyModel(col1='bar', col2=2, col3=False)
        o2.save()
        self.assertEquals(MyModel.objects.get(col2=1).col1, 'foo')
        self.assertEquals(MyModel.objects.get(col2=2).col1, 'bar')




# -*- coding: utf8 -*-

"""
This is a collection of utilities to setup the environment
for a standalone django script. This uses option parsers and
fake django settings.
"""

def settings(**kw):
    """
    This function returns the settings that are created for
    a simple sqlite3 database with a given file name. The
    settings are preconfigured so you can actually do normal
    chores with them.

    It uses sqlite3 as default for the database engine because
    that is based on a driver that will be preinstalled in
    modern python installations.

    You can pass in anything you want the settings to carry
    as named parameters - values from the parameter list will
    override potential library defaults.
    """
    from django.conf import settings

    if 'DATABASE_ENGINE' not in kw:
        kw['DATABASE_ENGINE'] = 'sqlite3'
    if 'INSTALLED_APPS' in kw:
        kw['INSTALLED_APPS'] = kw['INSTALLED_APPS'] + ('standalone',)
    else:
        kw['INSTALLED_APPS'] = (
            'standalone',
        )

    settings.configure(**kw)
    return settings




from setuptools import setup
from standalone import __version__

setup(
    name='django-standalone',
    version=__version__,
    description='use the Django ORM with standalone scripts',
    author='Georg Bauer',
    author_email='gb@rfc1437.de',
    url='http://bitbucket.org/rfc1437/django-standalone/',
    packages=['standalone'],
    scripts=['example_script.py'],
    long_description = file('README').read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Utilities",
    ],
    keywords = "django standalone script orm",
    license = 'BSD',
    install_requires = ['django'],
    test_suite = 'standalone.runtests.runtests',
)
